@@ -4,6 +4,7 @@ import re
 import sys
 import warnings
 from collections import deque
+from functools import partial
 from inspect import isclass
 from inspect import isfunction
 from inspect import isgenerator
@@ -12,21 +13,21 @@ from inspect import ismethod
 from inspect import ismethoddescriptor
 from inspect import ismodule
 from inspect import isroutine
-try:
-    from types import InstanceType
-except ImportError:
-    InstanceType = None
-from functools import partial
 from logging import getLogger
 
+from .utils import PY2
+from .utils import PY3
+from .utils import Sentinel
 from .utils import basestring
 from .utils import force_bind
 from .utils import logf
 from .utils import make_method_matcher
 from .utils import mimic
-from .utils import PY2
-from .utils import PY3
-from .utils import Sentinel
+
+try:
+    from types import InstanceType
+except ImportError:
+    InstanceType = None
 
 try:
     import __builtin__
@@ -40,7 +41,7 @@ except ImportError:
 
 
 __all__ = 'weave', 'Aspect', 'Proceed', 'Return', 'ALL_METHODS', 'NORMAL_METHODS', 'ABSOLUTELY_ALL_METHODS'
-__version__ = '1.3.3'
+__version__ = '1.4.1'
 
 logger = getLogger(__name__)
 logdebug = logf(logger.debug)
@@ -307,6 +308,7 @@ class Rollback(object):
     def __exit__(self, *_):
         for rollback in self._rollbacks:
             rollback()
+        del self._rollbacks[:]
 
     rollback = __call__ = __exit__
 
@@ -415,14 +417,12 @@ def weave(target, aspects, **options):
             _check_name(part)
 
         if len(parts) == 1:
-            __import__(part)
-            return weave_module(sys.modules[part], aspects, **options)
+            return weave_module(_import_module(part), aspects, **options)
 
         for pos in reversed(range(1, len(parts))):
             owner, name = '.'.join(parts[:pos]), '.'.join(parts[pos:])
             try:
-                __import__(owner)
-                owner = sys.modules[owner]
+                owner = _import_module(owner)
             except ImportError:
                 continue
             else:
@@ -464,16 +464,18 @@ def weave(target, aspects, **options):
         inst = target.__self__
         name = target.__name__
         logdebug("@ patching %r (%s) as instance method.", target, name)
-        func = getattr(inst, name)
+        func = target.__func__
         setattr(inst, name, _checked_apply(aspects, func).__get__(inst, type(inst)))
         return Rollback(lambda: delattr(inst, name))
     elif PY3 and isfunction(target):
         if bag.has(target):
             return Nothing
-        owner = __import__(target.__module__)
+        owner = _import_module(target.__module__)
         path = deque(target.__qualname__.split('.')[:-1])
+        print(owner, target.__module__)
         while path:
             owner = getattr(owner, path.popleft())
+            print(owner, )
         name = target.__name__
         logdebug("@ patching %r (%s) as a property.", target, name)
         func = owner.__dict__[name]
@@ -481,7 +483,7 @@ def weave(target, aspects, **options):
     elif PY2 and isfunction(target):
         if bag.has(target):
             return Nothing
-        return patch_module_function(__import__(target.__module__), target, aspects, **options)
+        return patch_module_function(_import_module(target.__module__), target, aspects, **options)
     elif PY2 and ismethod(target):
         if target.im_self:
             if bag.has(target):
@@ -489,7 +491,7 @@ def weave(target, aspects, **options):
             inst = target.im_self
             name = target.__name__
             logdebug("@ patching %r (%s) as instance method.", target, name)
-            func = getattr(inst, name)
+            func = target.im_func
             setattr(inst, name, _checked_apply(aspects, func).__get__(inst, type(inst)))
             return Rollback(lambda: delattr(inst, name))
         else:
@@ -631,7 +633,7 @@ def weave_class(klass, aspect, methods=NORMAL_METHODS, subclasses=True, lazy=Fal
         name = name or klass.__name__
         SubClass = type(name, (klass, Fabric), wrappers)
         SubClass.__module__ = klass.__module__
-        module = owner or __import__(klass.__module__)
+        module = owner or _import_module(klass.__module__)
         entanglement.merge(patch_module(module, name, SubClass, original=klass, aliases=aliases))
     else:
         original = {}
@@ -677,6 +679,11 @@ def _find_super_classes(klass):
                 yield k
 
 
+def _import_module(module):
+    __import__(module)
+    return sys.modules[module]
+
+
 def patch_module(module, name, replacement, original=UNSPECIFIED, aliases=True, location=None, **_bogus_options):
     """
     Low-level attribute patcher.
